package com.enoram.restify;

import java.util.logging.Logger;

import com.enoram.restify.server.RESTAPI;
import com.enoram.restify.server.RestRequestInfo;
import com.enoram.restify.server.codemodel.HttpSerializationReader;
import com.enoram.restify.server.codemodel.HttpSerializationWriter;
import com.enoram.restify.server.guice.GuiceRestify;
import com.enoram.restifycore.HttpRequestSerializationWriter;
import com.enoram.restifycore.HttpSerializationStreamReader;
import com.enoram.restifycore.SerializationReader;
import com.enoram.restifycore.SerializationWriter;
import com.google.inject.AbstractModule;
import com.google.inject.name.Names;



public abstract class AbstractRestModuleTest extends AbstractModule{
	private static final Logger log = Logger.getLogger(AbstractRestModuleTest.class.getName());


	@Override
	protected void configure() {
		  registerSerialization();
		  
		  registerResource();
		  
	}
	

	protected void registerSerialization() {
		bind(SerializationWriter.class).annotatedWith(HttpSerializationWriter.class)
		.to(HttpRequestSerializationWriter.class);
		
		bind(SerializationReader.class).annotatedWith(HttpSerializationReader.class)
		.to(HttpSerializationStreamReader.class);
		
	}
	protected abstract void registerResource();
	protected final void serveRest(Class servletClassTestImpl, Class restImplClazz){
		
		String apiNamed = GuiceRestify.getApiNamed(restImplClazz);
		//bind rest implementation class 
		bindRestImplClass(apiNamed, restImplClazz);
		
		//then serve for servlet
		
		//System.out.println("The REST API  running at http://[localhost] " + named);
		log.info("The REST API  running at http://[localhost] " + apiNamed);
		//Serve http://127.0.0.1:8888/_api/greet?[params]
		//also serve http://127.0.0.1:8888/_api/greet/?[params]
		// also serve http://127.0.0.1:8888/_api/greet/{named]/etc/etc/?params
		String servletPath = apiNamed+"/*";
	//	serve(apiNamed, servletPath).with(servletClass);
		
		bind(RestServiceTest.class).annotatedWith(Names.named(apiNamed))
		.to(servletClassTestImpl);
		
	}
	
	@SuppressWarnings("unchecked")
	protected final void bindProcessor(Class processorClazz){
		//bind 
		Class baseProcessorClazz = GuiceRestify.getBaseProcessor(processorClazz);
		
	//System.out.println("Base Processor: " + baseProcessorClazz.getSimpleName());
		
		bind(baseProcessorClazz).annotatedWith(Names.named(GuiceRestify.getUri(processorClazz)))
			.to(processorClazz);
		
	}
	protected final void bindRestImplClass(String apiNamed, Class restImplClazz){
		//bind 
		Class interfaceClass = GuiceRestify.getBaseProcessor(restImplClazz);
		
		String serviceIntfName = interfaceClass.getName();
		System.out.println("######### BIND  " + interfaceClass.getName() + "  of APINamed: : " + apiNamed + " to " + restImplClazz);
		
		bind(interfaceClass).annotatedWith(Names.named(apiNamed))
			.to(restImplClazz);
		
		// Get method, param along with annotation for params for later used for lookup
		RestRequestInfo reqInfo = GuiceRestify.getRestRequestInfo(restImplClazz);
		
		RESTAPI.register(apiNamed, serviceIntfName, reqInfo);
		
		
		
	}
}
