package com.enoram.restify;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.enoram.restify.server.REST;
import com.enoram.restify.server.RESTAPI;
import com.enoram.restify.server.RESTServletUtils;
import com.enoram.restify.server.RestRequest;
import com.enoram.restify.server.RestRequestContent;
import com.google.gwt.user.client.rpc.IncompatibleRemoteServiceException;
import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.SerializationException;
import com.google.gwt.user.server.rpc.RPC;
import com.google.gwt.user.server.rpc.RPCServletUtils;
import com.google.gwt.user.server.rpc.UnexpectedException;
import com.google.gwt.user.server.rpc.impl.LegacySerializationPolicy;
import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Key;
import com.google.inject.Singleton;
import com.google.inject.name.Names;

@Singleton
public class RestServiceServletTest implements RestServiceTest{

	private static final long serialVersionUID = 1L;

	@Inject
	private Injector injector;
	
	private String response;
	
	public void doPost(RestRequestContent restContent) throws Throwable{
		processPost(restContent);
	}
	
	public void doGet(RestRequestContent restContent) throws Throwable{
		doPost(restContent);
	}
	
	protected void processPost(RestRequestContent restContent) throws Throwable {


		// Let subclasses see the serialized request.
		//
		//onBeforeRequestDeserialized(requestPayload);

		// Invoke the core dispatching logic, which returns the serialized
		// result.
		//
		response = processCall(restContent);

		// Let subclasses see the serialized response.
		//
	//	onAfterResponseSerialized(responsePayload);

		// Write the response.
		//
	//	writeResponse(request, response, responsePayload);

	}

	/**
	 * Process a call originating from the given request. Uses the
	 * {@link RPC#invokeAndEncodeResponse(Object, java.lang.reflect.Method, Object[])}
	 * method to do the actual work.
	 * <p>
	 * Subclasses may optionally override this method to handle the payload in
	 * any way they desire (by routing the request to a framework component, for
	 * instance). The {@link HttpServletRequest} and {@link HttpServletResponse}
	 * can be accessed via the {@link #getThreadLocalRequest()} and
	 * {@link #getThreadLocalResponse()} methods.
	 * </p>
	 * This is public so that it can be unit tested easily without HTTP.
	 * 
	 * @param payload
	 *            the UTF-8 request payload
	 * @return a string which encodes either the method's return, a checked
	 *         exception thrown by the method, or an
	 *         {@link IncompatibleRemoteServiceException}
	 * @throws SerializationException
	 *             if we cannot serialize the response
	 * @throws UnexpectedException
	 *             if the invocation throws a checked exception that is not
	 *             declared in the service method's signature
	 * @throws RuntimeException
	 *             if the service method throws an unchecked exception (the
	 *             exception will be the one thrown by the service)
	 */
	public String processCall(RestRequestContent payload) throws SerializationException {
		try {

			String serviceIntfName = RESTAPI.getServiceIntfName(payload.getApiNamed());
			RestRequest req = REST.decodeRequest(payload, serviceIntfName);

//			RemoteService service = getServiceInstance(req.getMethod()
//					.getDeclaringClass());

			Class<?> clazz = REST.getClassFromSerializedName(serviceIntfName, this.getClass().getClassLoader());
			
			Object target = getServiceInstance(clazz, payload.getApiNamed());
			
			return REST.invokeAndEncodeResponse(target, req.getMethod(),
					req.getParameters(), LegacySerializationPolicy.getInstance());
		} catch (Exception ex) {
			
			return RPC.encodeResponseForFailure(null, ex);
		}finally{
			
	
			
		}
	}

	@SuppressWarnings({ "unchecked" })
	private Object getServiceInstance(Class<?> clazz, String apiNamed) {
		return (Object) injector.getInstance(Key.get(clazz, Names.named(apiNamed)));
	}
	
	

	/**
	 * Override this method to examine the serialized response that will be
	 * returned to the client. The default implementation does nothing and need
	 * not be called by subclasses.
	 * 
	 * @param serializedResponse
	 */
	protected void onAfterResponseSerialized(String serializedResponse) {
	}

	/**
	 * Override this method to examine the serialized version of the request
	 * payload before it is deserialized into objects. The default
	 * implementation does nothing and need not be called by subclasses.
	 * 
	 * @param serializedRequest
	 */
	protected void onBeforeRequestDeserialized(RestRequestContent serializedRequest) {
	}

	/**
	 * Override this method in order to control the parsing of the incoming
	 * request. For example, you may want to bypass the check of the
	 * Content-Type and character encoding headers in the request, as some
	 * proxies re-write the request headers. Note that bypassing these checks
	 * may expose the servlet to some cross-site vulnerabilities. Your
	 * implementation should comply with the HTTP/1.1 specification, which
	 * includes handling both requests which include a Content-Length header and
	 * requests utilizing <code>Transfer-Encoding:
	 * chuncked</code>.
	 * 
	 * @param request
	 *            the incoming request
	 * @return the content of the incoming request encoded as a string.
	 */
	protected RestRequestContent readContent(HttpServletRequest request)
			throws ServletException, IOException {
		return RESTServletUtils.readContentAsRestifyRest(request);
	}

	/**
	 * Determines whether the response to a given servlet request should or
	 * should not be GZIP compressed. This method is only called in cases where
	 * the requester accepts GZIP encoding.
	 * <p>
	 * This implementation currently returns <code>true</code> if the response
	 * string's estimated byte length is longer than 256 bytes. Subclasses can
	 * override this logic.
	 * </p>
	 * 
	 * @param request
	 *            the request being served
	 * @param response
	 *            the response that will be written into
	 * @param responsePayload
	 *            the payload that is about to be sent to the client
	 * @return <code>true</code> if responsePayload should be GZIP compressed,
	 *         otherwise <code>false</code>.
	 */
	protected boolean shouldCompressResponse(HttpServletRequest request,
			HttpServletResponse response, String responsePayload) {
		return RPCServletUtils
				.exceedsUncompressedContentLengthLimit(responsePayload);
	}

	@Override
	public String getResponse() {
		
		return response;
	}

}
