package com.enoram.restify.client;

import java.util.List;

import com.enoram.restify.client.rest.Named;
import com.enoram.restify.client.rest.RestApi;
import com.enoram.restify.client.rest.RestMethod;
import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * The client side stub for the RPC service.
 */
@RemoteServiceRelativePath("greet")
@RestApi(name="greet")
public interface GreetingService extends RemoteService {
	
	@RestMethod(httpMethod="GET", path="hello")
	String greetServer(@Named("greeting") String name) throws IllegalArgumentException;
	
	@RestMethod(path="register")
	String register(@Named("name")String name);
	
	@RestMethod(httpMethod="GET",path="names")
	List<String> listNames();
	
	/**
	 * path="name/{id}" is the same as "name?id=xx
	 * 
	 * @param id
	 * @return
	 */
	@RestMethod(path="name")
	String getName(@Named("id") int id);
	
	/**
	 * path: name/{id}/sub/{index}
	 * 
	 *  This is same as [uri]/name/sub?id=xx&start=nn&len=ll
	 *  
	 * @param id
	 * @param index
	 * @return
	 */
	@RestMethod(path="name/sub")
	String getSubName(@Named("id") int id, @Named("start") int start, @Named("len")int len);
	
	@RestMethod(path="greet/comment")
	Comment addComment(@Named("greetId") int id, @Named("comment") Comment comment);
	
	@RestMethod(path="void")
	void doVoidCall(@Named("name")String name);
	
	@RestMethod(path="comments")
	public List<Comment> comments();
}
