package com.enoram.restify.server;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import com.enoram.restify.client.Comment;
import com.enoram.restify.client.GreetingService;
import com.enoram.restify.shared.FieldVerifier;
import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * The server side implementation of the RPC service.
 */
@SuppressWarnings("serial")
public class GreetingServiceImpl extends RemoteServiceServlet implements
		GreetingService {

	public String greetServer(String input) throws IllegalArgumentException {
		// Verify that the input is valid. 
		if (!FieldVerifier.isValidName(input)) {
			// If the input is not valid, throw an IllegalArgumentException back to
			// the client.
			throw new IllegalArgumentException(
					"Name must be at least 4 characters long");
		}

		String serverInfo = "GAE SERVER ";
		String userAgent = "User-Agent";

		// Escape data from the client to avoid cross-site script vulnerabilities.
		input = escapeHtml(input);
		userAgent = escapeHtml(userAgent);

		return "Hello, " + input + "!I am running " + serverInfo
				+ ". It looks like you are using" + userAgent;
	}

	/**
	 * Escape an html string. Escaping data received from the client helps to
	 * prevent cross-site script vulnerabilities.
	 * 
	 * @param html the html string to escape
	 * @return the escaped string
	 */
	private String escapeHtml(String html) {
		if (html == null) {
			return null;
		}
		return html.replaceAll("&", "&amp;").replaceAll("<", "&lt;")
				.replaceAll(">", "&gt;");
	}

	private List<String> registrants = new ArrayList<String>();
	
	@Override
	public String register(String name) {
		registrants.add(name);
		return name;
	}

	@Override
	public List<String> listNames() {
		
		return registrants;
	}

	@Override
	public String getName(int id) {
		if(registrants.size()==0){
			registrants.add("RESTAPI");
		}
		return registrants.get(id);
	}

	@Override
	public String getSubName(int id, int start, int len) {
		if(registrants.size()==0){
			registrants.add("RESTAPI");
		}
		String name = registrants.get(id);
		return name.substring(start,len);
	}

	@Override
	public Comment addComment(int id, Comment comment) {
		Comment result = new Comment("SERVER: " + comment.getComment() );
		return result;
	}

	@Override
	public void doVoidCall(String name) {
		System.out.println("doVoidCall for "+ name);
		
	}

	@Override
	public List<Comment> comments() {
		Comment result1 = new Comment("SERVER Comment: " + System.currentTimeMillis());
		Comment result2 = new Comment("SERVER Comment: " + System.currentTimeMillis());
		Comment result3 = new Comment("SERVER Comment: " + System.currentTimeMillis());
		return Arrays.asList(result1, result2, result3);
	}
}
