package com.enoram.restify.server;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

public final class RESTAPI {

	private static Map<String, String> NAME_MAPS = new HashMap<String, String>();

	private static Map<String, RestRequestInfo> REQUEST_INFO_MAPS = new HashMap<String, RestRequestInfo>();

	public final static synchronized void register(String apiNamed,
			String serviceIntfName, RestRequestInfo reqInfo) {
		NAME_MAPS.put(apiNamed, serviceIntfName);
		REQUEST_INFO_MAPS.put(apiNamed, reqInfo);
	}

	public final static String getServiceIntfName(String apiNamed) {
		return NAME_MAPS.get(apiNamed);
	}

	public final static Method getMethod(String apiNamed, String methodPath) {

		RestRequestInfo info = REQUEST_INFO_MAPS.get(apiNamed);
		if (info == null) {
			throw new NullPointerException(
					"Cannot find RestRequestInfo for apiPath=" + apiNamed);
		}

		Method method = info.getMethods().get(methodPath);

		if (method == null) {
			throw new IllegalAccessError("Unable to get Method with apiPath: "
					+ apiNamed + " methodPath: " + methodPath);
		}

		return method;
	}

	/**
	 * The output of method params will be in String[] {@id int, @start int, int} 
	 * 
	 * Clazz:: com.enoram.restify.client.GreetingService
	 * 
	 * @register[POST] register(class java.lang.String)
	 * @name[POST] getName(@id int)
	 * @names[GET] listNames()
	 * @name/sub[POST] getSubName(@id int, @start int, int)
	 * 
	 * @param apiNamed
	 * @param methodPath
	 * @return
	 */
	public final static String[] getMethodParams(String apiNamed,
			String methodPath) {

		RestRequestInfo info = REQUEST_INFO_MAPS.get(apiNamed);
		if (info == null) {
			throw new NullPointerException(
					"Cannot find RestRequestInfo for apiPath=" + apiNamed);
		}

		String[] params = info.getParams().get(methodPath);

		if (params == null) {
			throw new IllegalAccessError(
					"Unable to get Method Params with apiPath: " + apiNamed
							+ " methodPath: " + methodPath);
		}

		return params;
	}

	private RESTAPI() {

	}

}
