package com.enoram.restify.server;

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

import com.google.gson.Gson;

public final class RestParam implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected String queryString;
	
	protected Map<String, String> params = new HashMap<String, String>();

	private String processorId;
	
	public RestParam() {
		super();
	}

	public RestParam(String queryString) {
		super();
		this.queryString = queryString;
		
	}

	/**
	 * Build Query String for url query based on parameters of this input param.
	 * 
	 * @return
	 */
	public String buildQueryString() {
		StringBuilder sb = new StringBuilder();
		
		try {
			Map<String, String> params = buildParams();
			
			for(String key: params.keySet()){
				sb.append(key + "=" + getValueEncoded(key)+ "&");
			}
			
			String result = sb.toString();
			if(result.endsWith("&")){
				result = result.substring(0, result.length()-1);
			}
			
			return result;
			
		} catch (UnsupportedEncodingException e) {
			
			e.printStackTrace();
		}
		
		
		
		
		
		return "";
	}
	/**
	 * Merge otherParams into this param object. After combining those params,
	 * you can call buildQueryString() 
	 * 
	 * @param otherParams
	 * @return
	 */
	public RestParam mergeParams(Map<String, String> otherParams){
		if(params==null){
			params = new HashMap<String, String>();
		}
		
		params.putAll(otherParams);
		
		return this;
				
	}
	
	public RestParam mergeEntityParams(Map<String, Object> entityParams){
	
		return this;
		
		
	}	
	
	protected String encoded(String param) throws UnsupportedEncodingException{
		return URLEncoder.encode(param,"UTF-8");
	}
	protected String decoded(String encoded) throws UnsupportedEncodingException{
		return URLDecoder.decode(encoded,"UTF-8");
	}
	
	protected Map<String, String> parseParam(String queryString) throws UnsupportedEncodingException{
		Map<String, String> result = new HashMap<String, String>();
		
		String[] ss = queryString.split("&");
		
		for(String s: ss){
			String[] pp = s.split("=");
			
			if(pp.length==2){
				String value = decoded(pp[1]);
			
				result.put(pp[0], value);
			}else{
				result.put(pp[0], "");
			}
		}
		
		
		return result;
		
	}

	public String getQueryString() {
		return queryString;
	}

	public void setQueryString(String queryString) {
		this.queryString = queryString;
	}

	public final Map<String, String> buildParams() throws UnsupportedEncodingException {
		if(params.isEmpty()){
			params = parseParam(queryString);
		}
		
		return params;
	}
	
	public final String getValue(String name) throws UnsupportedEncodingException{
		buildParams();
		return params.get(name);
	}
	
	public Object deserializeValue(String paramType, String paramName)throws UnsupportedEncodingException, ClassNotFoundException{
		String value = getValue(paramName);
		if(paramType.equals("int")){
			return Integer.parseInt(value);
		}
		
		if(paramType.equals("long")){
			return Long.parseLong(value);
		}
		
		if(paramType.equals("java.lang.String")){
			return value;
		}
		
		if(paramType.equals("java.lang.Long")){
			return Long.parseLong(value);
		}
		if(paramType.equals("java.lang.Integer")){
			return Integer.parseInt(value);
		}
		
		//Also you can have param name for json format: comment={comment-json-formatted-value}
		Class clazz = Class.forName(paramType);
		Gson gson = new Gson();
		Object jsonObject = gson.fromJson(value,
				clazz);

		return jsonObject;
		
			
		
	}
	
	public final String getValueEncoded(String name) throws UnsupportedEncodingException{
		buildParams();
		return encoded(params.get(name));
	}

	public String getProcessorId() {
		return processorId;
	}

	public void setProcessorId(String processorId) {
		this.processorId = processorId;
	}	
}
