package com.enoram.restify.server.codemodel;

import java.io.File;
import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.enoram.restify.server.GreetingServiceImpl;
import com.enoram.restify.server.ParamCallBack;
import com.enoram.restify.server.ParamParser;
import com.enoram.restify.server.RestRequestInfo;
import com.enoram.restify.server.guice.GuiceRestify;
import com.enoram.restifycore.HttpExecute;
import com.enoram.restifycore.RestApiException;
import com.enoram.restifycore.SerializationReader;
import com.enoram.restifycore.SerializationWriter;
import com.google.inject.Inject;
import com.sun.codemodel.JBlock;
import com.sun.codemodel.JClass;
import com.sun.codemodel.JClassAlreadyExistsException;
import com.sun.codemodel.JCodeModel;
import com.sun.codemodel.JDefinedClass;
import com.sun.codemodel.JDocComment;
import com.sun.codemodel.JExpr;
import com.sun.codemodel.JFieldVar;
import com.sun.codemodel.JMethod;
import com.sun.codemodel.JMod;
import com.sun.codemodel.JPackage;
import com.sun.codemodel.JType;
/**
 * Generate source code for client to be used to invoke REST API.
 * 
 * @author Nathan
 *
 */
public class GUICERESTFACTORY {

	private static final String RESTAPI_URI_NAME = "restApiUrl";
	private static final String HOST_URL_NAME = "hostUrl"; 
	
	private static final String HTTP_EXECUTE_INTERFACE_NAME = "httpExecute";
	private static final String WRITER_NAME = "serializationWriter";
	private static final String OBJ_WRITER_NAME = "serializationObjWriter";
	
	private static final String READER_NAME = "serializationReader";
	
	
	
	static RestRequestInfo requestInfo;
	
	public static void generate(Class<?> implClazz, String factroyPackage, String location){
		requestInfo = GuiceRestify.getRestRequestInfo(implClazz);
	    /* Creating java code model classes */
        JCodeModel jCodeModel = new JCodeModel();

        /* Adding packages here */
        JPackage jp = jCodeModel._package(factroyPackage);
        /* Giving Class Name to Generate */
        try {
        	String apiNamed = requestInfo.getApiNamed();
        	String apiNamedClass = formatAPIClassName(apiNamed);
			
        	JDefinedClass jc = jp._class(apiNamedClass);
			
            /* Adding class level coment */
            JDocComment jDocComment = jc.javadoc();
            jDocComment.add("A helper class to invoke REST API for " + apiNamedClass +       		" ");
			
            //class variable
            jc.field(JMod.PRIVATE, String.class, HOST_URL_NAME);
            jc.field(JMod.PRIVATE, String.class, RESTAPI_URI_NAME);
            
            createClassField(jc, HTTP_EXECUTE_INTERFACE_NAME, HttpExecute.class, Inject.class);
            
            //Serialize writer
            createClassField(jc, WRITER_NAME, SerializationWriter.class, Inject.class, HttpSerializationWriter.class);
            createClassField(jc, OBJ_WRITER_NAME, SerializationWriter.class, Inject.class, HttpSerializationWriter.class);
            
            //Serialize reader
            createClassField(jc, READER_NAME, SerializationReader.class, Inject.class, HttpSerializationReader.class);
            
            
            //CONNSTRUCTOR
            generateConstructorMethod(jCodeModel, jc, apiNamed, HOST_URL_NAME );
            
            //Generate GETTERS
            generateGetterMethod(jCodeModel, jc, JMod.PROTECTED, HTTP_EXECUTE_INTERFACE_NAME, HttpExecute.class);
            
            /* Adding method to the Class which is public static and returns com.somclass.AnyXYZ.class */
            Map<String, Method> methodMap = requestInfo.getMethods();
            for(String mpath: methodMap.keySet()){
            	
            	generateMethod(jCodeModel, jc, mpath, methodMap.get(mpath));
            	
            }
            

            /* Building class at given location */
            if(location !=null){
            	jCodeModel.build(new File(location));
            }else{
            	jCodeModel.build(new File("c:/temp/generated/src"));
            }
            
		} catch (Exception e) {

			e.printStackTrace();
		}
		
	}
	/**
	 * Format  /_api/greet to 'Greet'
	 * @param apiNamed
	 * @return
	 */
	static String formatAPIClassName(String apiNamed){
		
		String[] ss = apiNamed.split("/");
		String name = ss[2].substring(0,1).toUpperCase() ; 
		return name+ ss[2].substring(1)+"RestApi";
	}
 
	private static void createClassField(JDefinedClass jc, String fieldName, Class<?> clazz, Class<? extends Annotation>... annoClazzes){
		JFieldVar jField = jc.field(JMod.PRIVATE, clazz, fieldName);
        if(annoClazzes!=null){
        	for(Class<? extends Annotation> annoClazz: annoClazzes){
        		jField.annotate(annoClazz);
        	}
        }
	}
	
	private static void generateConstructorMethod(JCodeModel jCodeModel, JDefinedClass jc, String apiNamed, String hostUrlParamName) throws JClassAlreadyExistsException{
	    /* Adding method to the Class which is public static and returns com.somclass.AnyXYZ.class */
		  JMethod jmCreate =  jc.constructor(JMod.PUBLIC);
          /* Adding method body */
      
        /* Addign java doc for method */
        jmCreate.javadoc().add("Constructor to pass host URL ");

        //annotate

        
        /* Adding method body */
        JBlock jBlock = jmCreate.body();

        /* Defining method parameter */

         jmCreate.param(java.lang.String.class, hostUrlParamName);
         //@Named("rjo.official.bucketname") 
         jBlock.directStatement("this." + hostUrlParamName + " = " + hostUrlParamName + " ;");
        
         //set value for actual restAPIUrl
         jBlock.directStatement("//set value for actual restAPIUrl");
         
         jBlock.directStatement(RESTAPI_URI_NAME + " = " + hostUrlParamName + " + \""+ apiNamed + "\";");
         generateDirectStmtForValidPath(jBlock, RESTAPI_URI_NAME);
        /* Defining some class Variable in mthod body */
        
//        JClass jClassavpImpl = jCodeModel.ref(Comment.class);
//        JVar jvarAvpImpl = jBlock.decl(jClassavpImpl, "varName");
//        jvarAvpImpl.init(JExpr._new(jClassavpImpl));
//
//        
//        /* Adding some direct statement */             
//        jBlock.directStatement("varName.setCode(100);");

        /* returning varibalbe */        
     //s   jBlock._return(jvarAvpImpl);
	}
	
	private static void throwNewException(JCodeModel jCodeModel, JBlock jBlock, Class<? extends Exception> exClazz){
      JClass jClassavpImpl = jCodeModel.ref(exClazz);
      
      jBlock._throw(JExpr._new(jClassavpImpl));

//      jvarAvpImpl.init(JExpr._new(jClassavpImpl));
	}
	
	private static void generateDirectStmtForValidPath(JBlock jBlock , String paramName){
		jBlock.directStatement("if(!" + paramName + ".endsWith(" + "\"/\"))"+ "  this." + paramName + " += " + "\"/\"" +" ;");
		
	}
	private static void generateMethod(final JCodeModel jCodeModel, JDefinedClass jc, String mpath, Method method) throws JClassAlreadyExistsException, ClassNotFoundException{
	    /* Adding method to the Class which is public static and returns com.somclass.AnyXYZ.class */
        String methodName = method.getName();
        

        //returnClass.narrow(clazz)
        final JMethod jmCreate = jc.method(JMod.PUBLIC , method.getReturnType(), methodName);

        /* Adding java doc for method */
//        jmCreate.javadoc().add("Method Level Java Docs");

        String[]  params = requestInfo.getParams().get(mpath);
        if(params.length>0){
		 	   for(String param: params){
				   ParamParser parser = new ParamParser(param);
				   parser.parse(new ParamCallBack() {
					
					@Override
					public void performObject(String type, String pname) {
						// TODO Auto-generated method stub
						
					}
					
					@Override
					public void perform(String type, String pname) {
						JType jnativetype = getNativeTypeForCodeModel(jCodeModel, type);
						if(jnativetype !=null || type.equals("java.lang.String")){
							jmCreate.javadoc().addParam(pname);
						}else{
							jmCreate.javadoc().addParam(pname)
									.append(" param object with type " + type +" serialized in QueryString as JSON value" ); 
						}
						
					}
				});
			   }
        }
 

        /* Defining method parameter */
       generateMethodParam(jCodeModel, jmCreate, method);

        /* Defining some class Variable in mthod body */
        generateMethodBody(jCodeModel, jmCreate, method, mpath);
	}
	
	private static void generateGetterMethod(JCodeModel jCodeModel, JDefinedClass jc, int jMod, String fieldName, Class<?> returnClazz) throws JClassAlreadyExistsException, ClassNotFoundException{
	    /* Adding method to the Class which is public static and returns com.somclass.AnyXYZ.class */
        String methodName = "get"+fieldName.substring(0,1).toUpperCase()+fieldName.substring(1);
        
        
        //returnClass.narrow(clazz)
        JMethod jmethod = jc.method(jMod , returnClazz, methodName);
        
        /* Adding method body */
        JBlock jBlock = jmethod.body(); 
		
        jBlock.directStatement("return "  + fieldName + ";");
	}
	
	private static ParamString generateMethodParam(final JCodeModel jCodeModel, final JMethod jmethod, Method method) throws ClassNotFoundException{
		String[] params = GuiceRestify.getMethodParams(method);

		for(String sparam: params){

			ParamParser parser = new ParamParser(sparam);
			
			parser.parse(new ParamCallBack() {
				
				@Override
				public void performObject(String type, String pname) {
					// TODO Auto-generated method stub
					
				}
				
				@Override
				public void perform(String type, String pname) {
					JType jtype = getNativeTypeForCodeModel(jCodeModel, type);
					if(jtype!=null){
						jmethod.param(jtype, pname);
					
					}else{
			
						try {
							jmethod.param(Class.forName(type), pname);
						} catch (ClassNotFoundException e) {

							e.printStackTrace();
						}
					}
					
				}
			});
		}
		
		return null;
	}
	
	private static void generateMethodBody(JCodeModel jCodeModel, JMethod jmethod, Method method, String mpath) throws ClassNotFoundException{
	       /* Adding method body */
        final JBlock jBlock = jmethod.body(); 
		
		/* Defining some class Variable in mthod body */
//
//       JClass jClassavpImpl = jCodeModel.ref(HttpExecuteImpl.class);
//       JVar jvarAvpImpl = jBlock.decl(jClassavpImpl, "httpExecute");
//       jvarAvpImpl.init(JExpr._new(jClassavpImpl));

        
       
       /* Adding some direct statement */     
      
       jBlock.directStatement("String _url = " + RESTAPI_URI_NAME + "+" + ls(mpath)  + ";");
       
       String[]  params = requestInfo.getParams().get(mpath);
     
       if(params.length>0){
    	   jBlock.directStatement("try {");
    	   for(String param: params){
    		   ParamParser parser = new ParamParser(param);
    		   parser.parse(new ParamCallBack() {
				
				@Override
				public void performObject(String type, String pname) {
					jBlock.directStatement("    " + OBJ_WRITER_NAME+".writeObjectAsBodyContent(" + pname+ ");");
					
				}
				
				@Override
				public void perform(String type, String pname) {
					jBlock.directStatement("    " + WRITER_NAME+".writeString(" + ls(pname+"=") +");");
					jBlock.directStatement("    " +WRITER_NAME+".serialize("+pname+","+ls(type)+");");

					
				}
			});
    	   }
    	   
    	   jBlock.directStatement("}catch (com.enoram.restifycore.SerializationException e) {");
    	   throwNewException(jCodeModel, jBlock, RestApiException.class);
    	   jBlock.directStatement("} ");
    	   jBlock.directStatement("_url += " +ls("?") +"+"+ WRITER_NAME+".toString()"  + ";");   
	   
    	   //make sure to close the stream writer
    	   jBlock.directStatement(WRITER_NAME+".close()" + ";");   
    	   
       }
       
       //Generate Return statment for this method
       generateMethodReturnStmt(jCodeModel, jmethod, jBlock, method, mpath);
       


	}
	
	private static final String RESPONSE_NAME = "response_";
	
	private static void generateMethodReturnStmt(JCodeModel jCodeModel, JMethod jmethod, final JBlock jBlock, Method method, String mpath) throws ClassNotFoundException{
	    String httpMethod = GuiceRestify.getHttpMethod(method);
	      
		Class<?> clazzReturn = method.getReturnType();
		
		
		if(clazzReturn == Void.TYPE){
			if(httpMethod.equals("POST")){
		    	   jBlock.directStatement("getHttpExecute().executePostRequest(_url );");   
		     }else if(httpMethod.equals("GET")){
		    	   jBlock.directStatement("getHttpExecute().executeGetRequest(_url );");
		     }			
			return;
		}
	      
		if(httpMethod.equals("POST")){
	    	   jBlock.directStatement("String " + RESPONSE_NAME +" = getHttpExecute().executePostRequest(_url );");   
	     }else if(httpMethod.equals("GET")){
	    	   jBlock.directStatement("String " + RESPONSE_NAME +" = getHttpExecute().executeGetRequest(_url );");
	     }
		

		
		//jBlock.directStatement("return "+ RESPONSE_NAME +";");
		jBlock.directStatement("try {");
		
		jBlock.directStatement("return   (" + clazzReturn.getName() + ")" + READER_NAME+".deserialize("+ls(clazzReturn.getName())+","+RESPONSE_NAME+");");	
		
		 jBlock.directStatement("}catch (com.enoram.restifycore.SerializationException e) {");
  	   throwNewException(jCodeModel, jBlock, RestApiException.class);
  	   jBlock.directStatement("} ");
	}
		
	/**
	 * 
	 * @param s
	 * @return literal string quoted "s"
	 */
	private static String ls(String s){
		return "\"" + s + "\"";
	}

	   // Method to get JType based on any String Value
    private static JType getNativeTypeForCodeModel(JCodeModel jCodeModel, String type) {
        if (type.equals("Unsigned32")) {
            return jCodeModel.LONG;
        } else if (type.equals("long")) {
            return jCodeModel.LONG;
        } else if (type.equals("int")) {
            return jCodeModel.INT;
        } else {
            return null;
        }
    }
    
    private static class ParamString{
    	private String queryString="";
    	private String content;
    	
    	private List<String> params = new ArrayList<String>();
    	
    	
		public ParamString() {
			super();
		
		}
		public ParamString(String queryString, String content) {
			super();
			this.queryString = queryString;
			this.content = content;
		}
		public ParamString(String queryString) {
			super();
			this.queryString = queryString;
		}
	
		public void addParam(String pname){
			String token =  ls(pname+"=") +pname;
			params.add(token);
		}
		
		public void addQueryString(int index, String pname, String pvalue){
			
			queryString += (index==0?"":" + " + ls("&") + " + ") + ls(pname+"=") +" + " +pvalue;
		}
		public String getContent() {
			return content;
		}
		public void setContent(String content) {
			this.content = content;
		}
		public String getQueryString() {
			return queryString;
		}
		public void setQueryString(String queryString) {
			this.queryString = queryString;
		}
    }
    
    public static void main(String[] args) {
    
    	GUICERESTFACTORY.generate(GreetingServiceImpl.class, "com.enoram.restify.server.restapi.gen",
    	"C:\\temp\\restapi" 	);
	}
}
