package com.enoram.restify.server.codemodel;

import java.io.File;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.Map;

import com.enoram.restify.server.GreetingServiceImpl;
import com.enoram.restify.server.RestRequestInfo;
import com.enoram.restify.server.guice.GuiceRestify;
import com.enoram.restifycore.HttpExecuteImpl;
import com.sun.codemodel.JBlock;
import com.sun.codemodel.JClass;
import com.sun.codemodel.JClassAlreadyExistsException;
import com.sun.codemodel.JCodeModel;
import com.sun.codemodel.JDefinedClass;
import com.sun.codemodel.JDocComment;
import com.sun.codemodel.JExpr;
import com.sun.codemodel.JMethod;
import com.sun.codemodel.JMod;
import com.sun.codemodel.JPackage;
import com.sun.codemodel.JType;
import com.sun.codemodel.JVar;
/**
 * Generate source code for client to be used to invoke REST API.
 * 
 * @author Nathan
 *
 */
public class RESTFACTORY {

	private static final String RESTAPI_URI_NAME = "restApiUrl";
	private static final String HOST_URL_NAME = "hostUrl"; 
	public static void generate(Class<?> implClazz, String factroyPackage){
		RestRequestInfo requestInfo = GuiceRestify.getRestRequestInfo(implClazz);
	    /* Creating java code model classes */
        JCodeModel jCodeModel = new JCodeModel();

        /* Adding packages here */
        JPackage jp = jCodeModel._package(factroyPackage);
        /* Giving Class Name to Generate */
        try {
        	String apiNamed = requestInfo.getApiNamed();
        	String apiNamedClass = formatAPIClassName(apiNamed);
			
        	JDefinedClass jc = jp._class(apiNamedClass);
			
            /* Adding class level coment */
            JDocComment jDocComment = jc.javadoc();
            jDocComment.add("A helper class to invoke REST API for " + apiNamedClass +       		" ");
			
            //class variable
            jc.field(JMod.PRIVATE, String.class, HOST_URL_NAME);
            jc.field(JMod.PRIVATE, String.class, RESTAPI_URI_NAME);

            //CONNSTRUCTOR
            generateConstructorMethod(jCodeModel, jc, apiNamed, HOST_URL_NAME );

            

            
            /* Adding method to the Class which is public static and returns com.somclass.AnyXYZ.class */
            Map<String, Method> methodMap = requestInfo.getMethods();
            for(String mpath: methodMap.keySet()){
            	
            	generateMethod(jCodeModel, jc, mpath, methodMap.get(mpath));
            	
            }
            

            /* Building class at given location */
            jCodeModel.build(new File("c:/temp/generated/src"));

            
		} catch (Exception e) {

			e.printStackTrace();
		}
		
	}
	/**
	 * Format  /_api/greet to 'Greet'
	 * @param apiNamed
	 * @return
	 */
	static String formatAPIClassName(String apiNamed){
		
		String[] ss = apiNamed.split("/");
		String name = ss[2].substring(0,1).toUpperCase() ; 
		return name+ ss[2].substring(1)+"RestApi";
	}
 
	private static void generateConstructorMethod(JCodeModel jCodeModel, JDefinedClass jc, String apiNamed, String hostUrlParamName) throws JClassAlreadyExistsException{
	    /* Adding method to the Class which is public static and returns com.somclass.AnyXYZ.class */
		  JMethod jmCreate =  jc.constructor(JMod.PUBLIC);
          /* Adding method body */
      
        /* Addign java doc for method */
        jmCreate.javadoc().add("Constructor to pass host URL ");

        
        
        /* Adding method body */
        JBlock jBlock = jmCreate.body();

        /* Defining method parameter */

         jmCreate.param(java.lang.String.class, hostUrlParamName);
         
         jBlock.directStatement("this." + hostUrlParamName + " = " + hostUrlParamName + " ;");
         generateDirectStmtForValidPath(jBlock, hostUrlParamName);

         //set value for actual restAPIUrl
         jBlock.directStatement("//set value for actual restAPIUrl");
         
         jBlock.directStatement(RESTAPI_URI_NAME + " = " + hostUrlParamName + " + \""+ apiNamed + "\";");
         generateDirectStmtForValidPath(jBlock, RESTAPI_URI_NAME);
        /* Defining some class Variable in mthod body */
        
//        JClass jClassavpImpl = jCodeModel.ref(Comment.class);
//        JVar jvarAvpImpl = jBlock.decl(jClassavpImpl, "varName");
//        jvarAvpImpl.init(JExpr._new(jClassavpImpl));
//
//        
//        /* Adding some direct statement */             
//        jBlock.directStatement("varName.setCode(100);");

        /* returning varibalbe */        
     //s   jBlock._return(jvarAvpImpl);
	}
	
	private static void generateDirectStmtForValidPath(JBlock jBlock , String paramName){
		jBlock.directStatement("if(!" + paramName + ".endsWith(" + "\"/\")) "+ "  this." + paramName + " += " + "\"/\"" +" ;");
		
	}
	private static void generateMethod(JCodeModel jCodeModel, JDefinedClass jc, String mpath, Method method) throws JClassAlreadyExistsException, ClassNotFoundException{
	    /* Adding method to the Class which is public static and returns com.somclass.AnyXYZ.class */
        String methodName = method.getName();
        
        JClass returnClass =  jCodeModel.ref(method.getReturnType());
        
        Type type = method.getGenericReturnType();
        
        //returnClass.narrow(clazz)
        JMethod jmCreate = jc.method(JMod.PUBLIC , method.getReturnType(), methodName);

        /* Addign java doc for method */
        jmCreate.javadoc().add("Method Level Java Docs");

        
        
 

        /* Defining method parameter */
        generateMethodParam(jCodeModel, jmCreate, method);

        /* Defining some class Variable in mthod body */
        generateMethodBody(jCodeModel, jmCreate, method, mpath);
	}
	
	
	private static void generateMethodParam(JCodeModel jCodeModel, JMethod jmethod, Method method) throws ClassNotFoundException{
		
		String[] params = GuiceRestify.getMethodParams(method);
		String param;
		for(String sparam: params){
			param= sparam;
		
			//remove the "class " word
			param = param.replaceAll("class ", "");
			
			String[] ss = param.split(" ");

			if (ss.length < 2) {
					throw new IllegalAccessError(
							"Bad param. Expected 'name type' pair param");
			}
	
			if (ss[0].startsWith("@")) {
				String named = ss[0];
				named = named.substring(1); // remove @
				JType jtype = getTypeDetailsForCodeModel(jCodeModel, ss[1]);
				if(jtype!=null){
					jmethod.param(jtype, named);
				}else{
					
					jmethod.param(Class.forName(ss[1]), named);
				}
			}
		}
		
	}
	
	private static void generateMethodBody(JCodeModel jCodeModel, JMethod jmethod, Method method, String mpath) throws ClassNotFoundException{
	       /* Adding method body */
        JBlock jBlock = jmethod.body(); 
		
		/* Defining some class Variable in mthod body */
//
       JClass jClassavpImpl = jCodeModel.ref(HttpExecuteImpl.class);
       JVar jvarAvpImpl = jBlock.decl(jClassavpImpl, "httpExecute");
       jvarAvpImpl.init(JExpr._new(jClassavpImpl));

       
       /* Adding some direct statement */     
       String httpMethod = GuiceRestify.getHttpMethod(method);
       
       jBlock.directStatement("String _url = hostUrl + \""  +mpath + "\";");
       
       if(httpMethod.equals("POST")){
    	   jBlock.directStatement("return httpExecute.executePostRequest(_url );");   
       }else if(httpMethod.equals("GET")){
    	   jBlock.directStatement("return httpExecute.executeGetRequest(_url );");
       }
       


	}
		
		
	   // Method to get JType based on any String Value
    private static JType getTypeDetailsForCodeModel(JCodeModel jCodeModel, String type) {
        if (type.equals("Unsigned32")) {
            return jCodeModel.LONG;
        } else if (type.equals("long")) {
            return jCodeModel.LONG;
        } else if (type.equals("int")) {
            return jCodeModel.INT;
        } else if (type.equals("Integer64")) {
            return jCodeModel.LONG;
        } else if (type.equals("Enumerated")) {
            return jCodeModel.INT;
        } else if (type.equals("Float32")) {
            return jCodeModel.FLOAT;
        } else if (type.equals("Float64")) {
            return jCodeModel.DOUBLE;
        } else {
            return null;
        }
    }
    

    public static void main(String[] args) {
    
    	RESTFACTORY.generate(GreetingServiceImpl.class, "com.enoram.restify.server.restapi.gen");
	}
}
