package com.enoram.restify.server.guice;

import java.util.logging.Logger;

import javax.servlet.http.HttpServlet;

import com.enoram.restify.server.RESTAPI;
import com.enoram.restify.server.RestRequestInfo;
import com.enoram.restify.server.codemodel.HttpSerializationReader;
import com.enoram.restify.server.codemodel.HttpSerializationWriter;
import com.enoram.restifycore.HttpRequestSerializationWriter;
import com.enoram.restifycore.HttpSerializationStreamReader;
import com.enoram.restifycore.SerializationReader;
import com.enoram.restifycore.SerializationWriter;
import com.google.inject.name.Names;



public abstract class AbstractRestModule extends com.google.inject.servlet.ServletModule{
	private static final Logger log = Logger.getLogger(AbstractRestModule.class.getName());

	  @Override
	protected final void configureServlets() {

		  registerSerialization();
		  
		  registerResource();
		  
		 
	}



	protected void registerSerialization() {
		bind(SerializationWriter.class).annotatedWith(HttpSerializationWriter.class)
		.to(HttpRequestSerializationWriter.class);
		
		bind(SerializationReader.class).annotatedWith(HttpSerializationReader.class)
		.to(HttpSerializationStreamReader.class);
		
	}

	protected void registerResource() {
		
		
	}
	
	
	/**
	 * Using Guice serve for register servlet and bind the processor/class 
	 * 
	 * 		 bind(EntityProcessor.class).annotatedWith(Names.named(EntityPutProcessor.NAMED))
	        .to(EntityPutProcessor.class);
	 * 
	 * @param clazz
	 */
	protected final void serveAndBind(Class<? extends HttpServlet> servletClass, Class servletProcessorClazz, Class... clazzes){
		
		for(Class clazz: clazzes){
			bindProcessor(clazz);
		}
		//don't forget to bind Processor Servlet class
		bindProcessor(servletProcessorClazz);
		
		//then serve for servlet
		serve(GuiceRestify.getUri(servletProcessorClazz)).with(servletClass);
		
		
	}
	
	protected final void serveRest(Class<? extends HttpServlet> servletClass, Class restImplClazz){
		
		String apiNamed = GuiceRestify.getApiNamed(restImplClazz);
		//bind rest implementation class 
		bindRestImplClass(apiNamed, restImplClazz);
		
		//then serve for servlet
		
		//System.out.println("The REST API  running at http://[localhost] " + named);
		log.info("The REST API  running at http://[localhost] " + apiNamed);
		//Serve http://127.0.0.1:8888/_api/greet?[params]
		//also serve http://127.0.0.1:8888/_api/greet/?[params]
		// also serve http://127.0.0.1:8888/_api/greet/{named]/etc/etc/?params
		String servletPath = apiNamed+"/*";
		serve(apiNamed, servletPath).with(servletClass);
		
		
	}
	
	@SuppressWarnings("unchecked")
	protected final void bindProcessor(Class processorClazz){
		//bind 
		Class baseProcessorClazz = GuiceRestify.getBaseProcessor(processorClazz);
		
	//System.out.println("Base Processor: " + baseProcessorClazz.getSimpleName());
		
		bind(baseProcessorClazz).annotatedWith(Names.named(GuiceRestify.getUri(processorClazz)))
			.to(processorClazz);
		
	}
	protected final void bindRestImplClass(String apiNamed, Class restImplClazz){
		//bind 
		Class interfaceClass = GuiceRestify.getBaseProcessor(restImplClazz);
		
		String serviceIntfName = interfaceClass.getName();
		System.out.println("######### BIND  " + interfaceClass.getName() + "  to API Named: : " + apiNamed);
		
		bind(interfaceClass).annotatedWith(Names.named(apiNamed))
			.to(restImplClazz);
		
		// Get method, param along with annotation for params for later used for lookup
		RestRequestInfo reqInfo = GuiceRestify.getRestRequestInfo(restImplClazz);
		
		RESTAPI.register(apiNamed, serviceIntfName, reqInfo);
		
		
		
	}
}
