package com.enoram.restify.server.guice;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import com.enoram.restify.client.GreetingService;
import com.enoram.restify.client.rest.AsProcessor;
import com.enoram.restify.client.rest.AsResource;
import com.enoram.restify.client.rest.Named;
import com.enoram.restify.client.rest.RestApi;
import com.enoram.restify.client.rest.RestMethod;
import com.enoram.restify.server.GreetingServiceImpl;
import com.enoram.restify.server.RestRequestInfo;
import com.google.gson.Gson;

public class GuiceRestify {
	private static final Logger log = Logger.getLogger(GuiceRestify.class
			.getName());

	public GuiceRestify() {
		super();

	}

	@SuppressWarnings("rawtypes")
	public static String getUri(Class clazz) {
		Annotation[] annotations = clazz.getAnnotations();

		AsResource anno = (AsResource) clazz.getAnnotation(AsResource.class);

		if (anno == null) {
			AsProcessor named = (AsProcessor) clazz
					.getAnnotation(AsProcessor.class);
			return named.named();
		}

		String kind = anno.uri();
		return kind;

	}

	private static final String BASE_API_NAMED = "/_api";

	@SuppressWarnings("rawtypes")
	public static String getApiNamed(Class clazz) {
		Annotation[] annotations = clazz.getAnnotations();
		RestApi anno = null;
		if (annotations == null || annotations.length == 0) {
			Class interfaceClazz = getBaseProcessor(clazz);
			annotations = interfaceClazz.getAnnotations();
			anno = (RestApi) interfaceClazz.getAnnotation(RestApi.class);
		} else {
			anno = (RestApi) clazz.getAnnotation(RestApi.class);
		}

		String named = anno.name(); // ex: greet

		if (named.contains("/")) {
			throw new RuntimeException("@RestApi's name  cannot have '/'");
		}

		String apinamed = BASE_API_NAMED + "/" + anno.name(); // ex:
																// "/_api/greet"
		return apinamed;
	}

	/**
	 * Populate existing entity with object instance properties
	 * 
	 * serve(EntityPutProcessor.SERVLET_PATH).with(EntityPutV1ProcessorServlet.
	 * class);
	 * 
	 * 
	 * @param instance
	 * @param entity
	 * @return return this entity without creating new Entity object
	 */
	public static Class getBaseProcessor(Class clazz) {

		Class superClazz = clazz.getInterfaces()[0];

		return superClazz;
	}

	public static RestRequestInfo getRestRequestInfo(Class<?> implClazz) {

		Class intfClazz = getBaseProcessor(implClazz);
		String restApi = getApiNamed(intfClazz);

		RestRequestInfo reqInfo = new RestRequestInfo(restApi);

		Method[] methods = intfClazz.getMethods();

		System.out.println("Clazz:: " + intfClazz.getName() + "");

		Map<String, Method> methodMap = new HashMap<String, Method>();
		Map<String, String[]> paramMap = new HashMap<String, String[]>();
		for (Method method : methods) {
			String mpath = getMethodPath(method);
			
			if (mpath != null) { // only annotated by RestMethod, store it
				
				methodMap.put(mpath, method);
				paramMap.put(mpath, getMethodParams(method));

				System.out.println(")");
			}
		}

		reqInfo.setMethods(methodMap);
		reqInfo.setParams(paramMap);

		return reqInfo;
	}

	private static String getMethodPath(Method method) {

		RestMethod anno = method.getAnnotation(RestMethod.class);
		if (anno != null) {
			String mpath = anno.path(); 
			System.out.print("    [" + mpath + "]  @" + mpath + "[" + anno.httpMethod()+ "] " +method.getName()+"(");
			return mpath;
		}
		return null;
	}
	
	public static String getHttpMethod(Method method){
		RestMethod anno = method.getAnnotation(RestMethod.class);
		if (anno != null) {
			String mpath = anno.httpMethod(); 
			
			return mpath;
		}
		return null;		
	}
//	public static Class<?> getMethodGenericReturnType(Method method){
//		RestMethod anno = method.getAnnotation(RestMethod.class);
//		if (anno != null) {
//			Class<?>returnType= anno.genericReturnType(); 
//			
//			return returnType;
//		}
//		return Void.class;		
//	}
	public static String[] getMethodParams(Method method) {

		Type[] types = method.getGenericParameterTypes();
		String[] ss = new String[types.length];
		Annotation[][] annotations = method.getParameterAnnotations();
		//sString[] paramName = method.();
		int i=0;
		
		for (Type type : types) {
			String param ="";
			
			if(annotations[i].length>0 ){
				Named named = (Named) annotations[i][0];
				param = "@"+named.value()+" ";
				
			}else{ //get the param name
				
			}
			param += type;
			ss[i] =  param;
			
			System.out.print("" + ss[i] +(i<types.length-1?", ":"") );
			i++;
		}
		return ss;
	}

	@SuppressWarnings("rawtypes")
	private List<Field> getFields(Class clazz) {

		List<Field> fieldResult = new ArrayList<Field>();

		getFields(clazz, fieldResult, "private");

		return fieldResult;
	}

	/**
	 * Get the field of this class then iterate to this class.parentClazz to get
	 * all 'private' field .
	 * 
	 * 
	 * @param clazz
	 * @param fieldResult
	 */
	@SuppressWarnings("rawtypes")
	private void getFields(Class clazz, List<Field> fieldResult,
			String... modifiers) {
		Field fld[] = clazz.getDeclaredFields();

		for (Field field : fld) {

			int mod = field.getModifiers();

			// System.out.println("modifiers = " + Modifier.toString(mod) + " "
			// + field.getName());

			String modfy = Modifier.toString(mod);

			for (String modf : modifiers) {
				if (modfy.equals(modf)) {
					fieldResult.add(field);
				}
			}

		}

		Class superClazz = clazz.getSuperclass();
		if (superClazz != null) {
			getFields(superClazz, fieldResult, "private");
		}

	}

	boolean isNativeEntityType(Class fieldType) {
		if (fieldType == Integer.TYPE || fieldType == Integer.class
				|| fieldType == String.class || fieldType == Date.class) {
			return true;
		}
		return false;
	}

	public static void main(String[] args) {
		RestRequestInfo info = getRestRequestInfo(GreetingServiceImpl.class);
		
	}
	
}
