package com.enoram.restify.server.http;

import com.enoram.restify.BaseTest;
import com.enoram.restify.RestServiceTest;
import com.enoram.restify.server.RestRequestContent;
import com.enoram.restifycore.HttpExecute;
import com.google.inject.Inject;

/**
 * HttpExecute for Test enviroment implementation.
 * 
 * This doPost and doGet execution will call RestServiceTest that will act as Server.
 * 
 * 
 * @author Nathan
 *
 */
public class HttpExecuteTestImpl implements HttpExecute {

	private static int COUNTER=0;
	
	
	@Inject
	RestServiceTest restService;
	
	
	
	public HttpExecuteTestImpl() {
		super();
		System.out.println("HttpExecuteTestImpl - CREATED ::: " + ++COUNTER);;

	}

	@Override
	public String uploadFilePart(String baseUrl, String fileName) {
		//not yet implemented for TEST
		return null;
	}

	//_api/greet
	private static String[] parseApiNamedAndMethodName(String url){
		int index = url.indexOf("_api");
		if(index<0) 
			throw new RuntimeException("url must contain '/_api/' path. Url="+url);
		String validUrl = url.substring(index);
		
		String[] ss = validUrl.split("/");
		if(ss.length<1) throw new RuntimeException("url must contain at least '/_api/apiName/methodName' . Url="+url);
		
		String apiNamed = "/"+ ss[0]+"/"+ss[1];
		
		String methodName = ss[2];
		//append method name till the last 
		for(int i=3; i < ss.length; i++){
			methodName += "/"+ss[i];
		}
		
		if(methodName.contains("?")){
			methodName = methodName.substring(0, methodName.indexOf("?"));
		}
		
		return new String[]{apiNamed, methodName};
		
	}
	
	private static String parseQueryString(String url){
		int index = url.indexOf("?");
		return url.substring(index+1);
	}
	
	public static void main(String[] args) {
		String[] apiNamedMethod = parseApiNamedAndMethodName("http://localhost:999/_api/greet/hello?gretting=hello");
		String apiNamed = apiNamedMethod[0];
		String method = apiNamedMethod[1];
		
		System.out.println("api:: " + apiNamed + " m:: " + method);
		
	}
	
	
	@Override
	public String executePostRequest(String urlString) {
		System.out.println("URL="+urlString);
		System.out.println("++++++++++ Counter::: " + COUNTER);;
		
		///_api/greet
		String[] apiNamedMethod = parseApiNamedAndMethodName(urlString);
		String API_NAMED = apiNamedMethod[0];
		String METHOD_NAMED = apiNamedMethod[1];
		RestRequestContent reqContent = BaseTest.createRestRequestContent(API_NAMED, METHOD_NAMED, parseQueryString(urlString));
		try {
			restService.doPost(reqContent);
			String response = restService.getResponse();
			return response;
		} catch (Throwable e) {
		
			e.printStackTrace();
		}
		
		
		
		return null;
	}

	@Override
	public String executeGetRequest(String urlString) {
		return executePostRequest(urlString);
	}

	@Override
	public String encoded(String value) {
		return null;
	}

	@Override
	public String executePostWithBodyRequest(String json, String urlString) {
		// TODO Auto-generated method stub
		return null;
	}

}
