
package com.enoram.restify.server.restapi.gen;

import java.util.List;
import com.enoram.restify.client.Comment;
import com.enoram.restify.server.codemodel.HttpSerializationReader;
import com.enoram.restify.server.codemodel.HttpSerializationWriter;
import com.enoram.restifycore.HttpExecute;
import com.enoram.restifycore.RestApiException;
import com.enoram.restifycore.SerializationReader;
import com.enoram.restifycore.SerializationWriter;
import com.google.inject.Inject;


/**
 * A helper class to invoke REST API for GreetRestApi 
 * 
 */
public class GreetRestApi {

    private String hostUrl;
    private String restApiUrl;
    @Inject
    private HttpExecute httpExecute;
    @Inject
    @HttpSerializationWriter
    private SerializationWriter serializationWriter;
    @Inject
    @HttpSerializationWriter
    private SerializationWriter serializationObjWriter;
    @Inject
    @HttpSerializationReader
    private SerializationReader serializationReader;

    /**
     * Constructor to pass host URL 
     * 
     */
    public GreetRestApi(String hostUrl) {
        this.hostUrl = hostUrl ;
        //set value for actual restAPIUrl
        restApiUrl = hostUrl + "/_api/greet";
        if(!restApiUrl.endsWith("/"))  this.restApiUrl += "/" ;
    }

    protected HttpExecute getHttpExecute() {
        return httpExecute;
    }

    /**
     * 
     * @param greeting
     */
    public String greetServer(String greeting) {
        String _url = restApiUrl+"hello";
        try {
            serializationWriter.writeString("greeting=");
            serializationWriter.serialize(greeting,"java.lang.String");
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
        _url += "?"+serializationWriter.toString();
        serializationWriter.close();
        String response_ = getHttpExecute().executeGetRequest(_url );
        try {
        return   (java.lang.String)serializationReader.deserialize("java.lang.String",response_);
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
    }

    /**
     * 
     * @param name
     */
    public String register(String name) {
        String _url = restApiUrl+"register";
        try {
            serializationWriter.writeString("name=");
            serializationWriter.serialize(name,"java.lang.String");
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
        _url += "?"+serializationWriter.toString();
        serializationWriter.close();
        String response_ = getHttpExecute().executePostRequest(_url );
        try {
        return   (java.lang.String)serializationReader.deserialize("java.lang.String",response_);
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
    }

    /**
     * 
     * @param greetId
     * @param comment
     *      param object with type com.enoram.restify.client.Comment serialized in QueryString as JSON value
     */
    public Comment addComment(int greetId, Comment comment) {
        String _url = restApiUrl+"greet/comment";
        try {
            serializationWriter.writeString("greetId=");
            serializationWriter.serialize(greetId,"int");
            serializationWriter.writeString("comment=");
            serializationWriter.serialize(comment,"com.enoram.restify.client.Comment");
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
        _url += "?"+serializationWriter.toString();
        serializationWriter.close();
        String response_ = getHttpExecute().executePostRequest(_url );
        try {
        return   (com.enoram.restify.client.Comment)serializationReader.deserialize("com.enoram.restify.client.Comment",response_);
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
    }

    public List listNames() {
        String _url = restApiUrl+"names";
        String response_ = getHttpExecute().executeGetRequest(_url );
        try {
        return   (java.util.List)serializationReader.deserialize("java.util.List",response_);
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
    }

    /**
     * 
     * @param id
     */
    public String getName(int id) {
        String _url = restApiUrl+"name";
        try {
            serializationWriter.writeString("id=");
            serializationWriter.serialize(id,"int");
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
        _url += "?"+serializationWriter.toString();
        serializationWriter.close();
        String response_ = getHttpExecute().executePostRequest(_url );
        try {
        return   (java.lang.String)serializationReader.deserialize("java.lang.String",response_);
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
    }

    /**
     * 
     * @param name
     */
    public void doVoidCall(String name) {
        String _url = restApiUrl+"void";
        try {
            serializationWriter.writeString("name=");
            serializationWriter.serialize(name,"java.lang.String");
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
        _url += "?"+serializationWriter.toString();
        serializationWriter.close();
        getHttpExecute().executePostRequest(_url );
    }

    /**
     * 
     * @param id
     * @param start
     * @param len
     */
    public String getSubName(int id, int start, int len) {
        String _url = restApiUrl+"name/sub";
        try {
            serializationWriter.writeString("id=");
            serializationWriter.serialize(id,"int");
            serializationWriter.writeString("start=");
            serializationWriter.serialize(start,"int");
            serializationWriter.writeString("len=");
            serializationWriter.serialize(len,"int");
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
        _url += "?"+serializationWriter.toString();
        serializationWriter.close();
        String response_ = getHttpExecute().executePostRequest(_url );
        try {
        return   (java.lang.String)serializationReader.deserialize("java.lang.String",response_);
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
    }

    public List comments() {
        String _url = restApiUrl+"comments";
        String response_ = getHttpExecute().executePostRequest(_url );
        try {
        return   (java.util.List)serializationReader.deserialize("java.util.List",response_);
        }catch (com.enoram.restifycore.SerializationException e) {
        throw new RestApiException();
        } 
    }

}
