package com.enoram.restify.server.restapi.gen;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.junit.BeforeClass;
import org.junit.Test;

import com.enoram.restify.RestApiClientTest;
import com.enoram.restify.client.Comment;
import com.enoram.restify.client.GreetingService;
import com.google.inject.AbstractModule;
import com.google.inject.Guice;
import com.google.inject.Injector;
import com.google.inject.Module;
import com.google.inject.name.Names;

/**
 * A tester class to test basic GreetService REST API
 * 
 * @author Nathan
 * @see GreetingService
 * 
 */
public class TestGreetRestApi extends BaseTestRestApi {

	static Injector injector;

	@BeforeClass
	public static void  setupOnce(){
		
		BASE_MODULES.addAll(setupRestApiClientModules());
		injector = Guice.createInjector(BASE_MODULES); 
		
	}

	protected static List<Module> setupRestApiClientModules(){
		List<Module> modules = new ArrayList<Module>();
		
		modules.add(new RestApiClientTest());
		
		modules.add( new AbstractModule() {
			
			@Override
			protected void configure() {
			
			
				bind(GreetRestApi.class).to(GreetRestApiTest.class);
				
				bind(String.class)
		        .annotatedWith(Names.named("test.host"))
		        .toInstance("http://localhost");
			}
			}
			);
		
		
		
		return modules;
	}

	private GreetRestApi getGreetRestApi() {
		return injector.getInstance(GreetRestApi.class);
	}

	@Test
	public void doRegister() {
		String response = getGreetRestApi().register(
				"Register Hello from GreetRestApi");

		assertResponse(response);
	}

	@Test
	public void testGet(){
		String response = getGreetRestApi().getName(0);
		
		assertResponse(response);
	}
	
	@Test
	public void doGreetServer() {
		GreetRestApi restApi = getGreetRestApi();
		String response = restApi.greetServer("GreetRestApi");

		assertResponse(response);

		response = restApi.register("Register Hello from GreetRestApi 1234");

		assertResponse(response);

		response = restApi.getSubName(0, 0, 3);

		assertResponse(response);
	}

	@Test
	public void doAddComment() {
		Comment response = getGreetRestApi().addComment(0,
				new Comment("Comment From REST API Java Client"));

		assertResponse(response);
	}
	
	@Test
	public void testGetList(){
		List list = getGreetRestApi().comments();
		
		
		
	}
	
	private void assertResponse(Object response) {
		System.out.println(response);

		Assert.assertTrue(response != null);
		Assert.assertTrue(!response.toString().contains("//EX"));
	}

}
